/*++
  The material contained herein is not a license, either        
  expressly or impliedly, to any intellectual property owned    
  or controlled by any of the authors or developers of this     
  material or to any contribution thereto. The material         
  contained herein is provided on an "AS IS" basis and, to the  
  maximum extent permitted by applicable law, this information  
  is provided AS IS AND WITH ALL FAULTS, and the authors and    
  developers of this material hereby disclaim all other         
  warranties and conditions, either express, implied or         
  statutory, including, but not limited to, any (if any)        
  implied warranties, duties or conditions of merchantability,  
  of fitness for a particular purpose, of accuracy or           
  completeness of responses, of results, of workmanlike         
  effort, of lack of viruses and of lack of negligence, all     
  with regard to this material and any contribution thereto.    
  Designers must not rely on the absence or characteristics of  
  any features or instructions marked "reserved" or             
  "undefined." The Unified EFI Forum, Inc. reserves any         
  features or instructions so marked for future definition and  
  shall have no responsibility whatsoever for conflicts or      
  incompatibilities arising from future changes to them. ALSO,  
  THERE IS NO WARRANTY OR CONDITION OF TITLE, QUIET ENJOYMENT,  
  QUIET POSSESSION, CORRESPONDENCE TO DESCRIPTION OR            
  NON-INFRINGEMENT WITH REGARD TO THE TEST SUITE AND ANY        
  CONTRIBUTION THERETO.                                         
                                                                
  IN NO EVENT WILL ANY AUTHOR OR DEVELOPER OF THIS MATERIAL OR  
  ANY CONTRIBUTION THERETO BE LIABLE TO ANY OTHER PARTY FOR     
  THE COST OF PROCURING SUBSTITUTE GOODS OR SERVICES, LOST      
  PROFITS, LOSS OF USE, LOSS OF DATA, OR ANY INCIDENTAL,        
  CONSEQUENTIAL, DIRECT, INDIRECT, OR SPECIAL DAMAGES WHETHER   
  UNDER CONTRACT, TORT, WARRANTY, OR OTHERWISE, ARISING IN ANY  
  WAY OUT OF THIS OR ANY OTHER AGREEMENT RELATING TO THIS       
  DOCUMENT, WHETHER OR NOT SUCH PARTY HAD ADVANCE NOTICE OF     
  THE POSSIBILITY OF SUCH DAMAGES.                              
                                                                
  Copyright 2006, 2007, 2008, 2009, 2010 Unified EFI, Inc. All  
  Rights Reserved, subject to all existing rights in all        
  matters included within this Test Suite, to which United      
  EFI, Inc. makes no claim of right.                            
                                                                
  Copyright (c) 2010, Intel Corporation. All rights reserved.   
   
--*/
/*++

Module Name:

  DevicePathBBTestFunction.c

Abstract:

  Interface Function Test Cases of Device Path Protocol

--*/

#include "DevicePathBBTestMain.h"

EFI_GUID  gEfiDevicePathMessagingUartFlowControlGuid  = DEVICE_PATH_MESSAGING_UART_FLOW_CONTROL;

#if (EFI_SPECIFICATION_VERSION >= 0x00020000)
EFI_GUID  gEfiDevicePathMessagingSASGuid              = DEVICE_PATH_MESSAGING_SAS;
#endif

/**
 *  Entrypoint for EFI_DEVICE_PATH_PROTOCOL_GUID Path Node Conformance Test.
 *  @param This a pointer of EFI_BB_TEST_PROTOCOL.
 *  @param ClientInterface a pointer to the interface to be tested.
 *  @param TestLevel test "thoroughness" control.
 *  @param SupportHandle a handle containing protocols required.
 *  @return EFI_SUCCESS Finish the test successfully.
 */

//
// TDS 3.1
//
EFI_STATUS
BBTestDevicePathNodeConformanceAutoTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  )
{
  EFI_STANDARD_TEST_LIBRARY_PROTOCOL  *StandardLib;
  EFI_STATUS                          Status;
  EFI_DEVICE_PATH_PROTOCOL            *DevicePath;
  EFI_TEST_ASSERTION                  AssertionType;
  UINT16                              Type;
  UINT16                              SubType;
  UINT16                              Length;
  MEMMAP_DEVICE_PATH                  *MemMap;
  MAC_ADDR_DEVICE_PATH                *Mac;
  IPv4_DEVICE_PATH                    *IPv4;
  IPv6_DEVICE_PATH                    *IPv6;
  ATAPI_DEVICE_PATH                   *Atapi;
  UART_DEVICE_PATH                    *Uart;
  UART_FLOW_CONTROL_DEVICE_PATH       *UartFlow;
  VENDOR_DEVICE_PATH                  *Vendor;
#if (EFI_SPECIFICATION_VERSION >= 0x00020000)  
  ISCSI_DEVICE_PATH                   *iSCSI;
#endif
  VLAN_DEVICE_PATH                    *Vlan;
  HARDDRIVE_DEVICE_PATH               *Hd;
  CHAR16                              *DevStr;

  //
  // Get the Standard Library Interface
  //
  Status = gtBS->HandleProtocol (
                  SupportHandle,
                  &gEfiStandardTestLibraryGuid,
                  &StandardLib
                  );

  if (EFI_ERROR (Status)) {
    return Status;
  }

  DevicePath  = (EFI_DEVICE_PATH_PROTOCOL *) ClientInterface;

  DevStr      = DevicePathToStr (DevicePath);
  StandardLib->RecordMessage (
                StandardLib,
                EFI_VERBOSE_LEVEL_QUIET,
                L"\nVerifying device path: %s\n",
                DevStr
                );
  gtBS->FreePool (DevStr);

  while (!IsDevicePathEnd (DevicePath)) {
    Type    = (UINT16) DevicePathType (DevicePath);
    SubType = (UINT16) DevicePathSubType (DevicePath);
    Length  = (UINT16) DevicePathNodeLength (DevicePath);

    //
    // Assertion Point 3.1.2.2
    // Check End of Hardware Device Path: End This Device Path
    //
    if ((Type == 0x7F ) && (SubType == 0x01)) {
      if (Length == 4) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid001,
                    L"EFI_DEVICE_PATH_PROTOCOL - End of Hardware Device Path - End This Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.3
    // Check Hardware Device Path: PCI Device Path
    //
    else if ((Type == 1) && (SubType == 1)) {
      if (Length == 6) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid002,
                    L"EFI_DEVICE_PATH_PROTOCOL - Hardware Device Path - PCI Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.4
    // Check Hardware Device Path: PCCARD Device Path
    //
    else if ((Type == 1) && (SubType == 2)) {
      if (Length == 5) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid003,
                    L"EFI_DEVICE_PATH_PROTOCOL - Hardware Device Path - PCCARD Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.5
    // Check Hardware Device Path: Memory Mapped Device Path
    //
    else if ((Type == 1) && (SubType == 3)) {
      MemMap = (MEMMAP_DEVICE_PATH *) DevicePath;
      if ((Length == 24) &&
          (MemMap->MemoryType < EfiMaxMemoryType || MemMap->MemoryType > 0x7FFFFFFF) &&
          (MemMap->EndingAddress >= MemMap->StartingAddress)
          ) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid004,
                    L"EFI_DEVICE_PATH_PROTOCOL - Hardware Device Path - Memory Mapped Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.6
    // Check Hardware Device Path: Vendor Device Path
    //
    else if ((Type == 1) && (SubType == 4)) {
      if (Length >= 20) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid005,
                    L"EFI_DEVICE_PATH_PROTOCOL - Hardware Device Path - Vendor Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.7
    // Check Hardware Device Path: Controller Device Path
    //
    else if ((Type == 1) && (SubType == 5)) {
      if (Length == 8) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid006,
                    L"EFI_DEVICE_PATH_PROTOCOL - Hardware Device Path - Controller Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.8
    // Check ACPI Device Path: ACPI Device Path
    //
    else if ((Type == 2) && (SubType == 1)) {
      if (Length == 12) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid007,
                    L"EFI_DEVICE_PATH_PROTOCOL - ACPI Device Path - ACPI Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.9
    // Check ACPI Device Path: Expanded ACPI Device Path
    //
    else if ((Type == 2) && (SubType == 2)) {
      if (Length >= 19) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid008,
                    L"EFI_DEVICE_PATH_PROTOCOL - ACPI Device Path - Expanded ACPI Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }

#if (EFI_SPECIFICATION_VERSION >= 0x00020000)	
    //
    // Assertion Point 3.1.2.10
    // Check ACPI _ADR Device Path: ACPI _ADR Device Path
    //
    else if ((Type == 2) && (SubType == 3)) {
      if (Length >= 8) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid040,
                    L"EFI_DEVICE_PATH_PROTOCOL - ACPI _ADR Device Path - ACPI _ADR Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
#endif

    //
    // Assertion Point 3.1.2.11
    // Check Messaging Device Path: ATAPI Device Path
    //
    else if ((Type == 3) && (SubType == 1)) {
      Atapi = (ATAPI_DEVICE_PATH *) DevicePath;
      if ((Length == 8) &&
          (Atapi->PrimarySecondary == 0 || Atapi->PrimarySecondary == 1) &&
          (Atapi->SlaveMaster == 0 || Atapi->SlaveMaster == 1)
          ) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid009,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - ATAPI Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.12
    // Check Messaging Device Path: SCSI Device Path
    //
    else if ((Type == 3) && (SubType == 2)) {
      if (Length == 8) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid010,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - SCSI Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.13
    // Check Messaging Device Path: Fibre Channel Device Path
    //
    else if ((Type == 3) && (SubType == 3)) {
      if (Length == 24) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid011,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - Fibre Channel Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.14
    // Check Messaging Device Path: 1394 Device Path
    //
    else if ((Type == 3) && (SubType == 4)) {
      if (Length == 16) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid012,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - 1394 Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.15
    // Check Messaging Device Path: USB Device Path
    //
    else if ((Type == 3) && (SubType == 5)) {
      if (Length == 6) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid013,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - USB Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
#if (EFI_SPECIFICATION_VERSION >= 0x0002000A)		
    //
    // Add a new CheckPoint for SATA Device Path which belong to UEFI 2.1
    // Check SATA Device Path
    //
    else if ((Type == 3) && (SubType == 18)) {
      if (Length == 10) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid048,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - SATA Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }	
#endif	
    //
    // Assertion Point 3.1.2.16
    // Check Messaging Device Path: USB Device Path(WWID)
    //
    else if ((Type == 3) && (SubType == 16)) {
      if (Length >= 10) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid041,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - USB Device Path(WWID)",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.17
    // Check Messaging Device Path: Device Logical Unit
    //
    else if ((Type == 3) && (SubType == 17)) {
      if (Length == 5) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid042,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - USB Device Path(WWID)",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.18
    // Check Messaging Device Path: USB Device Path(Class)
    //
    else if ((Type == 3) && (SubType == 15)) {
      if (Length == 11) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid014,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - USB Class Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.19
    // Check Messaging Device Path: I2O Device Path
    //
    else if ((Type == 3) && (SubType == 6)) {
      if (Length == 8) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid015,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - I2O Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.20
    // Check Messaging Device Path: MAC Address Device Path
    //
    else if ((Type == 3) && (SubType == 11)) {
      Mac = (MAC_ADDR_DEVICE_PATH *)DevicePath;
      if (Length == 37) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }
      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid016,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - MAC Address Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d, IfType - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length,
                    Mac->IfType
                    );
    }
    //
    // Assertion Point 3.1.2.21
    // Check Messaging Device Path: IPv4 Device Path
    //
    else if ((Type == 3) && (SubType == 12)) {
      IPv4 = (IPv4_DEVICE_PATH *) DevicePath;
      if ((Length == 19) && (IPv4->StaticIpAddress == 0 || IPv4->StaticIpAddress == 1)) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid017,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - IPv4 Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.22
    // Check Messaging Device Path: IPv6 Device Path
    //
    else if ((Type == 3) && (SubType == 13)) {
      IPv6 = (IPv6_DEVICE_PATH *) DevicePath;
      if ((Length == 43) && (IPv6->StaticIpAddress == 0 || IPv6->StaticIpAddress == 1)) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid018,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - IPv6 Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Add a new CheckPoint for VLAN Device Path which belong to UEFI 2.3
    // Check Messaging Device Path: Vlan Device Path
    //
    else if ((Type == 3) && (SubType == 20)) {
      Vlan = (VLAN_DEVICE_PATH *) DevicePath;
      if (Vlan->VlanId > 4094 || Vlan->VlanId) {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      }
      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid052,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - Vlan Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, VlanId - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Vlan->VlanId
                    );      
    }
    //
    // Assertion Point 3.1.2.23
    // Check Messaging Device Path: InfiniBand Device Path
    //
    else if ((Type == 3) && (SubType == 9)) {
      if (Length == 48) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid019,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - InfiniBand Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.24
    // Check Messaging Device Path: UART Device Path
    //
    else if ((Type == 3) && (SubType == 14)) {
      Uart = (UART_DEVICE_PATH *) DevicePath;
      if ((Length == 19) &&
          (Uart->Parity >= 0x00 && Uart->Parity <= 0x05) &&
          (Uart->StopBits >= 0x00 && Uart->StopBits <= 0x03)
          ) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid020,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - UART Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    } else if ((Type == 3) && (SubType == 10)) {
      Vendor = (VENDOR_DEVICE_PATH *) DevicePath;
      //
      // Assertion Point 3.1.2.26
      // Check Messaging Device Path: UART Flow Control Messaging Path
      //
      if (CompareMem (&Vendor->Guid, &gEfiDevicePathMessagingUartFlowControlGuid, sizeof (EFI_GUID)) == 0) {
        UartFlow = (UART_FLOW_CONTROL_DEVICE_PATH *) DevicePath;
        if ((Length == 24) && 
            (UartFlow->FlowControlMap == 0x1 || 
             UartFlow->FlowControlMap == 0x2)) {
          AssertionType = EFI_TEST_ASSERTION_PASSED;
        } else {
          AssertionType = EFI_TEST_ASSERTION_FAILED;
        }

        StandardLib->RecordAssertion (
                      StandardLib,
                      AssertionType,
                      gDevicePathBBTestFunctionAssertionGuid022,
                      L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - UART Flow Control Messaging Path",
                      L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                      __FILE__,
                      (UINTN)__LINE__,
                      (UINTN)Type,
                      (UINTN)SubType,
                      (UINTN)Length
                      );
      }
#if (EFI_SPECIFICATION_VERSION >= 0x00020000)	  
      //
      // Assertion Point 3.1.2.27
      // Check Messaging Device Path: Serial Attached SCSI (SAS) Device Path
      //
      else if (CompareMem (&Vendor->Guid, &gEfiDevicePathMessagingSASGuid, sizeof (EFI_GUID)) == 0) {
        if (Length == 44) {
          AssertionType = EFI_TEST_ASSERTION_PASSED;
        } else {
          AssertionType = EFI_TEST_ASSERTION_FAILED;
        }

        StandardLib->RecordAssertion (
                      StandardLib,
                      AssertionType,
                      gDevicePathBBTestFunctionAssertionGuid043,
                      L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - Serial Attached SCSI (SAS) Device Path",
                      L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                      __FILE__,
                      (UINTN)__LINE__,
                      (UINTN)Type,
                      (UINTN)SubType,
                      (UINTN)Length
                      );
      }
#endif
	  //
      // Assertion Point 3.1.2.25
      // Check Messaging Device Path: Vendor-Defined Device Path
      //
      else {
        if (Length >= 20) {
          AssertionType = EFI_TEST_ASSERTION_PASSED;
        } else {
          AssertionType = EFI_TEST_ASSERTION_FAILED;
        }

        StandardLib->RecordAssertion (
                      StandardLib,
                      AssertionType,
                      gDevicePathBBTestFunctionAssertionGuid021,
                      L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path - Vendor-Defined Device Path",
                      L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                      __FILE__,
                      (UINTN)__LINE__,
                      (UINTN)Type,
                      (UINTN)SubType,
                      (UINTN)Length
                      );
      }
    }
#if (EFI_SPECIFICATION_VERSION >= 0x00020000)	
    //
    // Assertion Point 3.1.2.28
    // Check Messaging Device Path: iSCSI Device Path
    //
    else if ((Type == 3) && (SubType == 19)) {
      iSCSI = (ISCSI_DEVICE_PATH *) DevicePath;
      if ((Length >= 18) && (((iSCSI->LoginOption) & 0x0405) == 0)) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid044,
                    L"EFI_DEVICE_PATH_PROTOCOL - Messaging Device Path -  iSCSI Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
#endif	
    //
    // Assertion Point 3.1.2.29
    // Check Media Device Path: Hard Drive Media Device Path
    //
    else if ((Type == 4) && (SubType == 1)) {
      Hd = (HARDDRIVE_DEVICE_PATH *) DevicePath;
      if ((Length == 42) &&
          (Hd->MBRType == 0x01 || Hd->MBRType == 0x02) &&
          (Hd->SignatureType == 0x00 || Hd->SignatureType == 0x01 || Hd->SignatureType == 0x02)
          ) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid023,
                    L"EFI_DEVICE_PATH_PROTOCOL - Media Device Path - Hard Drive Media Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.30
    // Check Media Device Path: CD-ROM Media Device Path
    //
    else if ((Type == 4) && (SubType == 2)) {
      if (Length == 24) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid024,
                    L"EFI_DEVICE_PATH_PROTOCOL - Media Device Path - CD-ROM Media Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.31
    // Check Media Device Path: Vendor-Defined Media Device Path
    //
    else if ((Type == 4) && (SubType == 3)) {
      if (Length >= 20) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid025,
                    L"EFI_DEVICE_PATH_PROTOCOL - Media Device Path - Vendor-Defined Media Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.32
    // Check Media Device Path: File Path Media Device Path
    //
    else if ((Type == 4) && (SubType == 4)) {
      if (Length >= 4) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid026,
                    L"EFI_DEVICE_PATH_PROTOCOL - Media Device Path - File Path Media Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
    //
    // Assertion Point 3.1.2.33
    // Check Media Device Path: Media Protocol Device Path
    //
    else if ((Type == 4) && (SubType == 5)) {
      if (Length == 20) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid027,
                    L"EFI_DEVICE_PATH_PROTOCOL - Media Device Path - Media Protocol Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }
#if (EFI_SPECIFICATION_VERSION >= 0x0002000A)		
    //
    // Add a new CheckPoint for PIWG Firmware Volume which belong to UEFI 2.1
    // Check SATA Device Path
    //
    else if ((Type == 4) && (SubType == 6)) {
      if (Length >=4) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid049,
                    L"EFI_DEVICE_PATH_PROTOCOL - Media Device Path - PIWG Firmware Volume",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }	
    //
    // Add a new CheckPoint for PIWG Firmware File which belong to UEFI 2.1
    // Check SATA Device Path
    //
    else if ((Type == 4) && (SubType == 7)) {
      if (Length >=4) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid050,
                    L"EFI_DEVICE_PATH_PROTOCOL - Media Device Path - PIWG Firmware File",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }		
#endif	
    //
    // Assertion Point 3.1.2.34
    // Check BIOS Boot Specification Device Path
    //
    else if ((Type == 5) && (SubType == 1)) {
      if (Length >= 8) {
        AssertionType = EFI_TEST_ASSERTION_PASSED;
      } else {
        AssertionType = EFI_TEST_ASSERTION_FAILED;
      }

      StandardLib->RecordAssertion (
                    StandardLib,
                    AssertionType,
                    gDevicePathBBTestFunctionAssertionGuid028,
                    L"EFI_DEVICE_PATH_PROTOCOL - BIOS Boot Specification Device Path",
                    L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                    __FILE__,
                    (UINTN)__LINE__,
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    } else {
      StandardLib->RecordMessage (
                    StandardLib,
                    EFI_VERBOSE_LEVEL_QUIET,
                    L"Unknown Node(Type - %d, Subtype - %d, Length - %d)\n",
                    (UINTN)Type,
                    (UINTN)SubType,
                    (UINTN)Length
                    );
    }

    DevicePath = NextDevicePathNode (DevicePath);
  }
  //
  // Assertion Point 3.1.2.1
  // Check End of Hardware Device Path: End Entire Device Path
  //
  Type    = (UINT16) DevicePathType (DevicePath);
  SubType = (UINT16) DevicePathSubType (DevicePath);
  Length  = (UINT16) DevicePathNodeLength (DevicePath);

  if ((Type == 0x7F) && (SubType == 0xFF)) {
    if (Length == 4) {
      AssertionType = EFI_TEST_ASSERTION_PASSED;
    } else {
      AssertionType = EFI_TEST_ASSERTION_FAILED;
    }

    StandardLib->RecordAssertion (
                  StandardLib,
                  AssertionType,
                  gDevicePathBBTestFunctionAssertionGuid029,
                  L"EFI_DEVICE_PATH_PROTOCOL - End of Hardware Device Path - End Entire Device Path",
                  L"%a:%d:Type - %d, Subtype - %d, Length - %d",
                  __FILE__,
                  (UINTN)__LINE__,
                  (UINTN)Type,
                  (UINTN)SubType,
                  (UINTN)Length
                  );
  }

  return EFI_SUCCESS;
}
